# -*- coding: utf-8 -*-
# Copyright 2022 Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Author(s): Kristo Mela
"""
Created on Wed Oct  2 19:57:54 2019

Column base joint according to EN 1993-1-8

@author: kmela
"""

import math

import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib.lines as lines


    
from metku.eurocodes.en1993 import constants
from metku.eurocodes.en1993 import en1993_1_1
from metku.materials import steel_data
import metku.eurocodes.en1992.en1992_1_1 as en1992
import metku.eurocodes.en1992.constants as en1992_const
from metku.sections.steel.ISection import HEA
from metku.structures.steel.plates import RectPlate
from metku.materials.steel_data import Steel
from metku.eurocodes.en1993.en1993_1_8.en1993_1_8 import TStubEndPlate, BoltRow, ROW_OUTSIDE_BEAM_TENSION_FLANGE, FIRST_ROW_BELOW_BEAM_TENSION_FLANGE
from metku.eurocodes.en1993.en1993_1_8.en1993_1_8 import ROW_OUTSIDE_BEAM_TENSION_FLANGE, FIRST_ROW_BELOW_BEAM_TENSION_FLANGE, TENSION_ROW, COMPRESSION_ROW, OTHER_INNER_ROW

# Anchor bolts
    
HPM_L = {16:{"d":16,"As":157,"L":280,"washer":[40,6],"dh":38,"k":10, "material": "B500B"},
             20:{"d":20,"As":245,"L":350,"washer":[44,6],"dh":46,"k":12, "material": "B500B"},
             24:{"d":24,"As":352,"L":430,"washer":[56,6],"dh":55,"k":13,"material": "B500B"},
             30:{"d":30,"As":561,"L":500,"washer":[65,8],"dh":70,"k":15, "material": "B500B"},
             39:{"d":39,"As":976,"L":700,"washer":[90,10],"dh":90,"k":18, "material": "B500B"},
             }

class AnchorBolt:
    """ For anchor bolt data """
    
    def __init__(self,bolt_type=HPM_L[24]):
        """ Constructor
            input:
                bolt_type .. from a list of bolts above
                
            attributes:
                d .. diameter of the shank (mm)
                As .. stress area of the threaded shank (mm2)
                L .. length of the bolt (mm)
                washer .. [0] washer diameter (mm)
                          [1] washer thickness (mm)
                dh .. diameter of the head [mm]
                k .. thickness of the head (mm)
                material .. 
        """
        
        self.d = bolt_type["d"]
        self.As = bolt_type["As"]
        self.L = bolt_type["L"]
        self.washer = bolt_type["washer"]
        self.dh = bolt_type["dh"]
        self.k = bolt_type["k"]
        self.material = Steel(bolt_type["material"])

class ColumnBase:
    
    
    def __init__(self,column_profile, plate_edges, plate_thickness, anchor_bolts, bolt_x=None, bolt_p=100, concrete="C25/30"):
        """ Constructor
            Input:
                column_profile .. SteelSection class object
                plate_edges .. list of edge distances
                               [0] .. distance from left column flange to edge
                               [1] .. distance from right column flange to edge
                               [2] .. distance from top edge to column flange
                               [3] .. distance from bottom edge to column flange
                plate_thickness .. thickness of the base plate [mm]
                anchor_bolts
                        
        """

        self.column = column_profile
        self.beam = column_profile
        self._c = plate_edges
        h = column_profile.h
        b = column_profile.b
        hp = h + c[0] + c[1]
        bp = b + c[2] + c[3]
        base_plate = RectPlate(bp,hp,plate_thickness,material="S355")
        self.plate = base_plate
        self.anchor_bolt = anchor_bolts
        self.p = bolt_p
        if bolt_x is None:
            bolt_x = [0,0]
            bolt_x[0] = -0.5*h-0.5*c[0]
            bolt_x[1] = 0.5*h+0.5*c[1]
            
        self.xb = bolt_x
        self.concrete = en1992_const.Concrete(concrete)        
        self.Ac0 = 1.0
        self.Ac1 = 1.0
        self.flange_weld = 5.0
        self.web_weld = 5.0
        self.bolt_rows = []
        self.add_bolt_row(anchor_bolts, bolt_x[0], self.p)
        self.add_bolt_row(anchor_bolts, bolt_x[1], self.p)
        self.beta_j = 2.0/3.0
        
    
    def add_bolt_row(self,bolt,z,p):
        """ Input:
            bolt .. bolt type
            z .. position of bolts relative to centroid of column
                 z < 0: left side
                 z > 0: right side
            p .. distance between bolts
        """
        #(self,bolt,p,z,flange_loc=INNER_ROW, plate_loc=OTHER_INNER_ROW,joint=None, row_type=TENSION_ROW)
        
        if z < -0.5*self.column.h:
            ploc = ROW_OUTSIDE_BEAM_TENSION_FLANGE
            row_type = TENSION_ROW
        elif z < 0:
            ploc = FIRST_ROW_BELOW_BEAM_TENSION_FLANGE
            row_type = TENSION_ROW
        else:
            ploc = OTHER_INNER_ROW
            row_type = COMPRESSION_ROW
        
        new_row = BoltRow(bolt,p,z,plate_loc=ploc,joint=self,row_type=row_type)
        
        self.bolt_rows.add(new_row)
    
    @property
    def fcd(self):
        """ Design compression strength of concrete """        
        return self.concrete.fcd()
        #return en1992.fcd(en1992.concrete[self.concrete]["])

    @property
    def fjd(self):
        """ Design compression strength of concrete """                     
        # Strength of the foundation
        return self.beta_j*self.fcd        

    @property
    def tp(self):
        """ Thickness of end plate """
        return self.plate.t
        #return en1992.fcd(en1992.concrete[self.concrete]["])

    @property
    def fyp(self):
        """ Yield strength of end plate """
        return self.plate.material.fy
        #return en1992.fcd(en1992.concrete[self.concrete]["])
        
    @property
    def c(self):
        """ Length 'c' of EN 1993-1-8, Eq. (6.5) """
        return self.tp*math.sqrt(self.fyp/3/self.fjd)
    
    
    def beff(self,c):
        """ Effective width """
        return 2*c+self.column.tf
    
    
    def leff(self,c):
        """ Effective length """
        return 2*c+self.column.b
    
    def bolt_coord(self):
        """ Returns list of tuples that are the xy-coordinates of the bolts """
        coords = []
        
        ypos = 0.5*self.p
        yneg = -ypos
        for xb in self.xb:    
            coords.append((xb,yneg))
            coords.append((xb,ypos))
            
        return coords
    
    def Fc_pl_Rd(self,verb=False):
        """ Compression strength of T-stub 
            EN 1993-1-8, Eq. (6.4)
        """
        c = self.c
        beff = self.beff(c)
        leff = self.leff(c)
        fjd = self.fjd
        
        FRd = fjd*beff*leff
        
        if verb:
            print("Compression resistance of T-stub:")
            print(f"c = {c:4.2f} mm")
            print(f"beff = {beff:4.2f} mm")
            print(f"fjd = {fjd:4.2f} MPa")
            print(f"FC,Rd = {FRd*1e-3:4.2f} kN")
        
        return FRd

    def Fc_fc_Rd(self,verb=False):
        """ Column flange and web in compression 
            EN 1993-1-8 6.2.6.7
        """
        McRd = self.column.MRd
        Fc = McRd/(self.column.h-self.column.tf)
        
        if verb:
            print("Column flange and web in compression")
            print(f"McRd = {McRd*1e-6:4.2f} kNm")
            print(f" h = {self.column.h:4.2f} mm")
            print(f" tf = {self.column.tf:4.2f} mm")
            print(f" Fc_fc_Rd = {Fc*1e-3:4.2f} kN")
        
        return Fc
    
    def FCRd(self,verb=False):
        """ Compression strength of the connection """
        
        FcplRd = self.Fc_pl_Rd(verb)
        FcfcRd = self.Fc_fc_Rd(verb)
        
        FC = min(FcplRd,FcfcRd)
        
        return FC
    
    def Ft_ep_Rd(self,verb=False):
        """ Tension side: bending of base plate.
            EN 1993-1-8, 6.2.6.5. Base plate is treated as end plate.
        """
        
        ex = 0.5*self.plate.h+self.xb[0]
        mx = 0.5*self.column.h+0.8*self.flange_weld*math.sqrt(2) + self.xb[0]
        
        T = TStubEndPlate(self.bolt_rows[0],
                          material=self.plate.material,
                          tf=self.plate.t,
                          emin=ex,
                          m=mx,
                          e=ex,
                          w=self.p,
                          bp=self.plate.b)
        
        F12Rd = T.F_T_12_Rd()
        F3Rd = T.F_T_3_Rd()
        
        
    
    def draw(self, name=""):
        """ Draw the connection """
        fig, ax = plt.subplots(1,2)
        ymax = 150
        
        self.column.draw(axes=ax[1])
        
        # Side view:
        # Origin is located at the bottom of the steel profile center line
        hp, bp, tp = self.plate.h, self.plate.b, self.plate.t
        base_plate = patches.Rectangle((-0.5*hp, -tp), width=hp, height=tp, fill=False, hatch='\\')
        
        base_plate_plan = patches.Rectangle((-0.5*hp, -0.5*bp), width=hp,
                                       height=bp, fill=False, hatch='//')
        
        
        # Draw bolts
        dbolt = self.anchor_bolt.d
        lbolt = self.anchor_bolt.L
        left_bolt = patches.Rectangle((self.xb[0]-0.5*dbolt, -tp-lbolt), width=dbolt,
                                       height=lbolt+tp, fill=True)
        
        right_bolt = patches.Rectangle((self.xb[1]-0.5*dbolt, -tp-lbolt), width=dbolt,
                                       height=lbolt+tp, fill=True)
        """
        bot_flange = patches.Rectangle((-0.5*self.bb, 0), width=self.bb,
                                       height=self.tb, fill=False, hatch='\\')
        web = patches.Rectangle((-0.5*self.tw, self.tb), width=self.tw,
                                height=self.hw, fill=False, hatch='\\')
    
        top_flange = patches.Rectangle((-0.5*self.bt, self.h-self.tt),
                                       width=self.bt, height=self.tt,
                                       fill=False, hatch='\\')
        """
        ax[0].add_patch(base_plate)
        h = self.column.h
        tf = self.column.tf
        ax[0].vlines([-0.5*h,-0.5*h+tf,0.5*h,0.5*h-tf],0,ymax)
        ax[0].vlines(0,0,ymax,linestyles='dashdot')
        ax[0].add_patch(left_bolt)
        ax[0].add_patch(right_bolt)
        ax[0].set_xlim(-0.5*hp, 0.5*hp)
        if self.anchor_bolt is None:
            ymin = -self.plate.t
        else:
            ymin = -self.anchor_bolt.L
        
        ax[0].set_ylim(ymin, ymax)
    
        
        ax[0].set_aspect('equal')
        
        xtick = ax[0].get_xticks()
        #print(xtick)
        
        """ Draw connection from above """
        ax[1].add_patch(base_plate_plan)
        
        # Draw column profile
        
        
        # Draw bolts
        xy_bolts = self.bolt_coord()

        for xy in xy_bolts:
            ax[1].add_patch(patches.Circle(xy,radius=0.5*dbolt,linestyle='solid'))

        
        ax[1].set_xticks(xtick)
        #ax[1].set_xlim(-0.5*hp, 0.5*hp)
        ax[1].set_ylim(-0.5*hp, 0.5*hp)
        ax[1].set_aspect('equal')
        
        return ax
        
if __name__ == '__main__':
    
    col = HEA(240)
    #w = col.b + 20
    #d = col.h + 140
    #plate = RectPlate(width=d,depth=d,thickness=30,material="S355")
        
    
    #conc = en1992.concrete["C25/30"]
    
    col.Ned = -263.22e3
    col.Med[0] = 39.57e6
    col.Ved[1] = 16.70e3
    
    c = [100,100,50,50]
    tp = 40
    bolts = AnchorBolt(HPM_L[24])

    joint = ColumnBase(col,plate_edges = c,\
                       plate_thickness = tp,\
                       anchor_bolts = bolts,\
                       bolt_p = 100,\
                       concrete="C30/37")
        
    ax = joint.draw()
    
    #concrete = en1992_const.Concrete("C25/30")
    #fcd = concrete.fcd
    #beta_j = 2.0/3.0
    # Strength of the foundation
    #fjd = beta_j*fcd
    
    """
    print("fcd = {0:4.2f} MPa".format(joint.fcd))
    print("fjd = {0:4.2f} MPa".format(joint.fjd))
    
    # Weld sizes
    joint.flange_weld = 6
    joint.web_weld = 6
    
    NEd = abs(joint.column.Ned)
    MEd = joint.column.Med
    h = joint.column.h
    tf = joint.column.tf
    NR = 0.5*NEd + MEd/(h-tf)
    NL = 0.5*NEd - MEd/(h-tf)
    
    print("NR = {0:4.2f}; NL ={1:4.2f}".format(NR,NL))
    
    e1 = 50.0
    p = 180.0
    
    if NL < 0:
        zR = 0.5*(h-tf)
        zL = 0.5*h+e1        
        print("zR = {0:4.2f} mm ; zL ={1:4.2f}".format(zR,zL))

    z = zR+zL    
    print("z = {0:4.2f} mm".format(z))
    
    NR_Ed = NEd*zL/z + MEd/z
    NL_Ed = NEd*zR/z - MEd/z
    
    print("NR_Ed = {0:4.2f} kN".format(NR_Ed*1e-3))
    print("NL_Ed = {0:4.2f} kN".format(NL_Ed*1e-3))
    
    c = joint.c
    
    print("c = {0:4.2f} mm".format(c))
    
    FCRd = joint.FCRd()
    Fc_fc_Rd = joint.Fc_fc_Rd()
    
    print("FCRd = {0:4.2f} kN".format(FCRd*1e-3))
    print("Fc_fc_Rd = {0:4.2f} kN".format(Fc_fc_Rd*1e-3))
    
    m = e1-0.8*math.sqrt(2)*joint.flange_weld
    outstand_top = 110
    d_bolt = 24
    
    Fbolt_Rd = 0.9*550*352/1.25
    print("Fbolt_Rd = {0:4.2f} kN".format(Fbolt_Rd*1e-3))
    
    e = outstand_top + 0.5*d_bolt
    ex = c-e1
    
    leff_cp = min([2*math.pi*m,math.pi*m+p,math.pi*m+2*e])
    leff_nc = min([4*m+1.25*e,e+2*m+0.625*ex,0.5*(joint.column.b+2*c),0.5*p+2*m+0.625*ex])
    
    print("m = {0:4.2f} mm".format(m))
    print(leff_cp)
    print(leff_nc)
    
    leff_1 = min(leff_nc,leff_cp)
    Mpl_1_Rd = 0.25*leff_1*joint.tp**2*joint.fyp
    FT_12_Rd = Mpl_1_Rd/m
    
    print("Mpl_1_Rd = {0:4.2f} kNm".format(Mpl_1_Rd*1e-6))
    print("FT_12_Rd = {0:4.2f} kN".format(FT_12_Rd*1e-3))
    
    F_t_wb_Rd = leff_1*joint.column.tw*joint.column.fy
    print("Ft_wb_Rd = {0:4.2f} kN".format(F_t_wb_Rd*1e-3))
    
    
    
    #joint.fcd
    
    """


# -*- coding: utf-8 -*-
# Copyright 2022 Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Author(s): Kristo Mela
"""
Created on Tue Apr 19 10:14:37 2022

Raami functions for exporting to various programs

@author: kmela
"""


class AbaqusOptions:
    """ Storage for ABAQUS export options """
    
    def __init__(self,**kwargs):
        """
        

        Parameters
        ----------
        **kwargs : TYPE
            DESCRIPTION.

        Returns
        -------
        None.

        """
        
        self.x_monitor = 0
        self.n_monitored = 1
        self.mpc = []
        self.ecc_elements = []
        self.top_gap_elements = []
        self.bottom_gap_elements = []
        
        self.elsets = {}
        
        # Individual members to be included as element sets
        self.included_members = []
        
        attr = self.__dict__.keys()
        
        for key, val in kwargs.items():
            if key in attr:
                setattr(self,key,val)


# Author(s): Kristo Mela
# This source code is licensed under the MIT license. See LICENSE in the repository root directory.
# Copyright 2022 Kristo Mela
# -*- coding: utf-8 -*-
"""
Created on Fri Jan  7 11:33:47 2022

Portal truss: two columns and a truss

@author: kmela
"""

import matplotlib.pyplot as plt

from metku.raami.raami import Raami
from metku.raami.frame_member import FrameMember, SteelFrameMember, MultiSpanSteelMember
from metku.raami.frame_node import FrameNode
from metku.raami.frame_loads import PointLoad, LineLoad, LoadIDs
from metku.raami.frame_supports import XYHingedSupport, FixedSupport
from metku.raami.raami_plane_truss import SlopedTruss

from metku.sections.steel.ISection import HEA, HEB, IPE

class PortalTruss(Raami):
    """ Class for portal frame trusses """


    def __init__(self,span=20000,height_eaves=10000,column=None,truss=None,name="Portal Truss"):
        
        super().__init__(name)
                
        self.L = span
        self.height = height_eaves
        self.column_profile = column
        self.truss = truss
        self.columns = []
    
        self.bottom_chord_to_column = False
    
        # Create nodes for column bases
        self.add(FrameNode([0,0]))
        self.add(FrameNode([self.L,0]))
    
    def __repr__(self):
        
        return f"{self.__name__}: height = {self.height:.2f} mm, span = {self.L:.2f} mm"
    
    @property
    def col_length(self):
        """ total length of the column """
        return self.height
    
    @property
    def col_profile(self):
        """ cross-section of the profiles """
        if len(self.columns) == 0:
            p = self.column_profile
        else:
            p = self.columns[0].cross_section
        return p
    
    @col_profile.setter
    def col_profile(self,val):
        """ set cross-section of the profiles """
        self.column_profile = val
        
        for col in self.columns:
            col.cross_section = val    
        
    def weight_detailed(self,verb=False):
        """ Returns detailed weight 
            The weight of truss and columns are separated.
        """
        
        Wtop, Wbottom, Wbraces = self.truss.weight_detailed(False)
        
        Wcols = 2*self.columns[0].weight()
        
        Wtruss = Wtop + Wbottom + Wbraces
        Wtot = Wtruss + Wcols
        
        if verb:
            print(f"Portal Truss weight: {Wtot:.2f} kg")
            print(f"Weight of columns (total): {Wcols:.2f} kg ({Wcols/Wtot*100:.1f}%)")            
            print(f"Truss weight: {Wtruss:.2f} kg ({Wtruss/Wtot*100:.1f}%)")            
            print(f"Top Chord: {Wtop:.2f} kg")
            print(f"Bottom Chord: {Wbottom:.2f} kg")
            print(f"Braces: {Wbraces:.2f} kg")
        
        return Wtop, Wbottom, Wbraces
    
    
    def add(self, this):
        """
        Adds object to truss
        :param this: object to be added
        :return:
        """
        # COLUMN
        if (isinstance(this, SteelFrameMember) or isinstance(this, MultiSpanSteelMember)) and this.mtype == 'column':
            self.columns.append(this)
            this.mem_id = len(self.members)
            self.members["COL" + str(this.mem_id)] = this
            this.frame = self
            
            if isinstance(this, MultiSpanSteelMember):
                for mem in this.members:
                    mem.frame = self
        else:
            super().add(this)
    
    def create_columns(self,profile=HEA(260),hinges=[False,False]):
        """ creates columns """
        if not self.truss is None:
            # Truss has already been created
            
            if self.bottom_chord_to_column:
                # Bottom chord is connected to the column
                self.add(MultiSpanSteelMember([self.nodes[0],self.truss.bottom_nodes[0],self.truss.top_nodes[0]],
                                             profile,mem_type='column',nel=4,hinges=hinges,sway=True))                    
                self.add(MultiSpanSteelMember([self.nodes[1],self.truss.bottom_nodes[-1],self.truss.top_nodes[-1]],profile,
                                                          mem_type='column',nel=4,hinges=hinges,sway=True))
            else:
                self.add(SteelFrameMember([self.nodes[0],self.truss.top_nodes[0]],profile,
                                                      mem_type='column',nel=4,hinges=hinges,lcr=[2.0,1.0],sway=True))
            
                self.add(SteelFrameMember([self.nodes[1],self.truss.top_nodes[-1]],profile,
                                                      mem_type='column',nel=4,hinges=hinges,lcr=[2.0,1.0],sway=True))
            
    
    def create_truss(self, topology='K', ndiv=4, truss_type='simple', **kwargs):
        """ Makes the truss based on user data """
        
        if truss_type == 'simple':
            # Simple truss means that the bottom chord is not connected to the
            # column.
            try:
                dx1 = kwargs['dx']
                dx2 = kwargs['dx']
            except:
                dx1 = 1500
                dx2 = 1500
        else:
            # Non-simple truss means that the bottom chord is connected to the column
            dx1 = 0
            dx2 = 0
        
        try:
            first = kwargs['first_diagonal_up']
        except:
            first = True
        
        if dx1 == 0:
            self.bottom_chord_to_column = True
        
        self.truss = SlopedTruss(L1=0.5*self.L,L2=0.5*self.L,dx1=dx1,dx2=dx2,**kwargs)
        
        
        self.truss.origin = (0,self.height)
        self.truss.generate_topology(topology,ndiv,first_diagonal_up=first,edge_verticals=False)
        self.truss.generate_joints()
        self.truss.fem = self.fem
        self.truss.symmetry()
        
        self.member_groups = self.truss.member_groups
        
        # Add the truss nodes to the frame:
        n0 = len(self.nodes)        
        
        for node in self.truss.nodes:
            node.node_id += n0
            self.nodes.append(node)
        
        for mem in self.truss.members.values():
            self.add(mem)
        #self.members = self.truss.members
    
    def generate_supports(self,col_base="fixed"):
        """ Creates supports to the truss """
        
        if col_base == 'fixed':
            self.add(FixedSupport(self.nodes[0]))
            self.add(FixedSupport(self.nodes[1]))

        elif col_base == 'pinned':
            self.add(XYHingedSupport(self.nodes[0]))
            self.add(XYHingedSupport(self.nodes[1]))

    def generate_fem(self,truss_eccentricity=False,column_eccentricity=False):
        """ Creates the finite element model """
        
        
        # Generate finite element model of the truss        
        
        if truss_eccentricity:
            self.truss.generate_fem("en1993")
        else:
            self.truss.generate_fem("no_eccentricity")
        
        
        for node in self.nodes[:2]:
            if self.dim == 2:
                newNode = self.fem.add_node(node.x,node.y)
            else:
                newNode = self.fem.add_node(node.x,node.y,node.z)
            
            node.fem_node = newNode
       
        
        # Generate finite elements for the columns        
        for member in self.columns:            
            member.generate_elements(self.fem)
        
        # Generate supports
        for supp in self.supports.values():
            supp.add_support(self.fem)

        # Set nodal degrees of freedom        
        self.fem.nodal_dofs()
        
        
    def generate_uniform_load(self,part='truss',q=-20,chord="top",load_id=LoadIDs['ULS'],ltype='live'):
        
        if part == 'truss':
            if not (load_id in self.load_ids):
                self.load_ids.append(load_id)
                
            self.truss.generate_uniform_load(q,chord,load_id,ltype)
    
    def plot(self,print_text=True, show=True,
             loads=True, color=False, axes=None, save=False, mem_dim=False):
        
        if axes is None:
            fig, ax = plt.subplots(1)
        else:
            ax = axes
        
        self.truss.plot(print_text=print_text,show=show,
                        loads=loads,color=color,axes=ax,save=False,mem_dim=mem_dim)
        
        for col in self.columns:
            col.plot(print_text=print_text, c=color, axes=ax, mem_dim=mem_dim)
        
        for support in self.supports.values():
            node_coord = support.node.coords
            if support.dofs == [-1, -1, -1] or support.dofs == [1, 1, 1] or support.dofs == [0, 1, 2]:
                marker = 's'
            elif support.dofs == [1]:
                marker = '^'
            elif support.dofs == [0]:
                marker = '>'
            else:
                marker = 'D'
            ax.scatter(node_coord[0], node_coord[1], s=50, c='k',
                        marker=marker)
        

        # if self.truss:
        #    self.truss.plot(show=False, print_text=print_text, color=color)
        if loads:
            self.plot_loads()

if __name__ == "__main__":
    
    from timeit import default_timer as timer
    
    p = PortalTruss()
    
    p.create_truss(topology='K',ndiv=3,h2=2000,h1=1500,dx=000,first_diagonal_up=True)
    p.truss.top_chord[0].add_hinge(0)
    p.truss.top_chord[-1].add_hinge(1)
    if p.bottom_chord_to_column:
        p.truss.bottom_chord[0].add_hinge(0)
        p.truss.bottom_chord[-1].add_hinge(1)
        
    p.generate_uniform_load('truss',q=-25)
    #p.truss.generate_supports()
    p.create_columns(hinges=[False,False])
    p.generate_supports()
    p.symmetry()
    
        
    
    #start = timer()
    print(p.weight())
    #end = timer()
    #print(end-start)
    #p.weight_detailed(True)
    #Wtop, Wbottom, Wbraces = p.truss.weight_detailed(True)
    p.generate_fem(truss_eccentricity=False)
        
    #print(p.weight())
    p.structural_analysis(load_id=p.load_ids[0],support_method="REM")
    #p.optimize_members(verb=True)
    p.bmd(scale=20,load_id=p.load_ids[0],loads=False)
    #p.fem.draw()
    #p.plot()
    
        